/**
 * Interactive charts widget
 * @type {Class}
 */

!function($){
	window.WIcharts = new Class({

		Implements: [Options, Events],

		options: {
			/**
			 * @var {Array} List of chart colors (may be used for branding)
			 */
			colors: ["#4572A7", "#89A54E", "#AA4643", "#80699B", "#3D96AE", "#DB843D", "#92A8CD", "#A47D7C", "#B5CA92"],
			/**
			 * @var {Array} List of chart params
			 * (!) Defines the lower logics and probably should not be changed
			 */
			params: ['company', 'period', 'rate'],
			/**
			 * Different types of representing the data
			 * @var {Object} index Separate chart, which can be grabbed by groups
			 * @var {String} index.type Highcharts series type http://api.highcharts.com/highcharts#series
			 * @var {String} index.fixed Which param will be fixed (from options.params)
			 * @var {String} index.axis Which param will be used for axis (from options.params)
			 * @var {String} index.vary Which param will be varied (from options.params)
			 * @var {String} index.measure Type of measure representation absolute/relative
			 * @var {String} index.compare Which param will be compared (from options.params)
			 * @var {Boolean} index.redraw Force full chart redraw (workaround for not-perfect draw logics)
			 * @var {Boolean} index.skipNegative This type of chart cannot handle negative values, so skip it
			 */
			charts: {
				0: {type: 'line', fixed: 'company', axis: 'period', vary: 'rate', measure: 'absolute', compare: 'period'},
				1: {type: 'column', fixed: 'company', axis: 'rate', vary: 'period', measure: 'absolute', compare: 'period'},
				2: {type: 'column', fixed: 'company', axis: 'period', vary: 'rate', measure: 'absolute', compare: 'period'},
				10: {type: 'line', fixed: 'company', axis: 'period', vary: 'rate', measure: 'relative', compare: 'period'},
				11: {type: 'column', fixed: 'company', axis: 'rate', vary: 'period', measure: 'relative', compare: 'period'},
				20: {type: 'line', fixed: 'rate', axis: 'period', vary: 'company', measure: 'absolute', compare: 'period'},
				21: {type: 'pie', fixed: 'rate', axis: 'company', vary: 'period', measure: 'relative', compare: 'period', redraw: true},
				22: {type: 'stackedbar', fixed: 'rate', axis: 'period', vary: 'company', measure: 'relative', compare: 'period', redraw: true},
				31: {type: 'column', fixed: 'period', axis: 'rate', vary: 'company', measure: 'absolute', compare: 'company'},
				32: {type: 'column', fixed: 'period', axis: 'company', vary: 'rate', measure: 'absolute', compare: 'company'},
				33: {type: 'stackedbar', fixed: 'period', axis: 'rate', vary: 'company', measure: 'relative', compare: 'company', redraw: true},
				41: {type: 'column', fixed: 'period', axis: 'company', vary: 'rate', measure: 'relative', compare: 'company'},
				42: {type: 'column', fixed: 'period', axis: 'rate', vary: 'company', measure: 'relative', compare: 'company'},
				43: {type: 'stackedbar', fixed: 'period', axis: 'rate', vary: 'company', measure: 'relative', compare: 'company', redraw: true}
			},
			groups: {
				/**
				 * Different types of groups
				 * @var {Function} index.condition Function to check if current group should be used.
				 * (!) Do not change keys to keep order of conditions checks
				 * @var {Object} index.controls Defines presense and behaviour of inner controls in selected group
				 * @var {Function} index.controls.onClick Onclick event
				 * @var {Function} index.controls.activeOn Condition of control active state
				 */
				0: {
					// 1 company / 1 unit / N years
					condition: function(active){ return (active.company.length == 1) && (active.units.length == 1); },
					controls: {
						line: {
							onClick: function(active, opt){ this._renderChart(0); },
							activeOn: function(active, opt){ return (opt.type == 'line'); }
						},
						column: {
							onClick: function(active, opt){ this._renderChart(1); },
							activeOn: function(active, opt){ return (opt.type == 'column'); }
						},
						toggle: {
							onClick: function(active, opt){ this._renderChart((this._chartType == 1) ? 2 : 1)},
							showOn: function(active, opt){ return (opt.type == 'column'); }
						}
					},
					charts: [0,1,2]
				},
				1: {
					// 1 company / N units
					condition: function(active){ return (active.company.length == 1); },
					controls: {
						line: {
							onClick: function(active, opt){ this._renderChart(10); },
							activeOn: function(active, opt){ return (opt.type == 'line'); }
						},
						column: {
							onClick: function(active, opt){ this._renderChart(11); },
							activeOn: function(active, opt){ return (opt.type == 'column'); }
						}
					},
					charts: [10,11]
				},
				2: {
					// N companies / 1 rate
					condition: function(active){ return (active.rate.length == 1); },
					controls: {
						line: {
							onClick: function(){ this._renderChart(20); },
							activeOn: function(active, opt){ return (opt.type == 'line'); }
						},
						pie: {
							onClick: function(){ this._renderChart(21); },
							activeOn: function(active, opt){ return (opt.type == 'pie'); }
						},
						stackedbar: {
							onClick: function(){ this._renderChart(22); },
							activeOn: function(active, opt){ return (opt.type == 'stackedbar'); }
						}
					},
					charts: [20,21,22]
				},
				3: {
					// N companies / N rates / 1 unit
					condition: function(active){ return (active.units.length == 1); },
					controls: {
						column: {
							onClick: function(){ this._renderChart(31); },
							activeOn: function(active, opt){ return (opt.type == 'column'); }
						},
						stackedbar: {
							onClick: function(){ this._renderChart(33); },
							activeOn: function(active, opt){ return (opt.type == 'stackedbar'); }
						},
						toggle: {
							onClick: function(){ this._renderChart((this._chartType == 31) ? 32 : 31)},
							showOn: function(active, opt){ return (opt.type == 'column'); }
						}
					},
					charts: [31,32,33]
				},
				4: {
					// N companies / N rates / N units
					condition: function(active){ return true; },
					controls: {
						column: {
							onClick: function(){ this._renderChart(41); },
							activeOn: function(active, opt){ return (opt.type == 'column'); }
						},
						stackedbar: {
							onClick: function(){ this._renderChart(43); },
							activeOn: function(active, opt){ return (opt.type == 'stackedbar'); }
						},
						toggle: {
							onClick: function(){ this._renderChart((this._chartType == 41) ? 42 : 41)},
							showOn: function(active, opt){ return (opt.type == 'column'); }
						}
					},
					charts: [41,42,43]
				}
			},
			/**
			 * @var {Object} Selectors for dom elements
			 */
			dom: {
				chart    : '.w-icharts-chart',
				xlsexport: '.w-icharts-xlsexport'
			},
			/**
			 * @var {Int} Max series on one chart
			 */
			seriesMax: 5,
			/**
			 * @var {Boolean} Show chart legend
			 */
			legend: true,

			/**
			 * @var {Number} Radius of a pie chart
			 */
			pieRadius: 80,

			/**
			 * @var {Boolean} Show data labels by default?
			 */
			dataLabels: true,

			/**
			 * @var {Number} gridLineWidth for Y-axis
			 */
			gridLineWidthY: 1,

			/**
			 * @var {Boolean} Hide Y-axis labels by default?
			 */
			hideYAxis: false,

			/**
			 * @var {Number} tickLength for X-axis
			 */
			tickLengthX: 5,

			/**
			 * @var {Number} tickLength for Y-axis
			 */
			tickLengthY: 5,

			/**
			 * @var {Number} Chart padding
			 */
			spacingTop: 10,
			spacingRight: 10,
			spacingBottom: 15,
			spacingLeft: 10,

			/**
			 * @var {String} plotOptions style
			 */
			labelsStyle: '',

			/**
			 * @var {String} X-axis labels style
			 */
			labelsXStyle: '',

			/**
			 * @var {Number} X-axis labels position
			 */
			labelsYX: 15,

			/**
			 * @var {Number} Y-axis labels position
			 */
			labelsYY: -15,

			/**
			 * @var {Number} X-axis labels rotation
			 */
			rotationLabelsX: 0,

			/**
			 * @var {Number} Y-axis labels rotation
			 */
			rotationLabelsY: 315,

			/**
			 * @var {String} Current active language
			 */
			lang: 'ru',

			/**
			 * @var {Object} Translations
			 */
			i18n: {
				ru: {pp: 'п.п.'},
				en: {pp: 'p.p.'}
			}
		},

		dom: {
			wrapper: {},
			rate: {},
			company: {},
			period: {},
			type: {}
		},

		/**
		 * @var {Object} Active params. Grouped by param name and stored in order of adding
		 */
		active: {
			rate: [],
			company: [],
			period: [],
			type: []
		},

		// Comparison parameter
		compare: false,

		/**
		 * @var {Object} Chart data
		 */
		data: {},

		/**
		 * @var {Array} Predefined color schema
		 */
		_colors: {},

		initialize: function(container, data, options)
		{
			this.setOptions(options);

			this.dom.container = $(container);

			this.data = data;

			// Data wrappers
			this.dom.wrapper = {};
			$.each(this.options.params, function(index, param){
				this.dom.wrapper[param] = this.dom.container.find('[data-wrapper="'+param+'"]');
			}.bind(this));

			// Attach controls
			this.attach(this.dom.container);

			// Grab dom elements
			$.each(this.options.dom, function(key, selector){
				this.dom[key] = this.dom.container.find(selector);
			}.bind(this));

			// Grab controls
			this.dom.controls = {};
			$.each(this.dom.container.find('[data-control]'), function(index, elm){
				var controlType = $(elm).data('control');
				if (this.dom.controls[controlType] !== undefined) return;
				this.dom.controls[$(elm).data('control')] = this.dom.container.find('[data-control="'+controlType+'"]');
			}.bind(this));

			// dataLabels toggler
			if (this.dom.controls['dataLabels'] !== undefined){
				this.dom.controls['dataLabels'].on('click', function(){
					this.options.dataLabels = ! this.options.dataLabels;
					this.dom.controls['dataLabels'][this.options.dataLabels?'addClass':'removeClass']('active');
					this._renderChart(this._chartType);
				}.bind(this));
			}

			// Chart export function
			this.dom.container.find('.i-export').each(function(index, elm){
				var exportType = $(elm).data('type');
				$(elm).on('click', function(){
					this.chart.exportChart({
						type: exportType
					});
				}.bind(this));
			}.bind(this));

			// MaxSeries is limited by number of colors
			this.options.seriesMax = Math.min(this.options.seriesMax, this.options.colors.length);

			this.chart = new Highcharts.Chart({
				chart: {
					renderTo: this.dom.chart.get(0)
				},
//			colors: $.extend([], this.options.colors),
				credits: {enabled: false},
				title: {text: ''},
				legend: {enabled: this.options.legend},
				xAxis: {title: {text: ''}},
				yAxis: {
					title: {text: (this.options.hideYAxis) ? null : ''},
					showEmpty: false
				}
			});

			this._group = false;

			this.render();
		},

		/**
		 * Attach contols inside the container element
		 * @param {Element} container
		 */
		attach: function(container)
		{
			$.each(this.options.params, function(paramIndex, param){
				this.dom[param] = {};
				$.each(container.find('[data-'+param+']'), function(elmIndex, elm){
					var value = parseInt($(elm).data(param));
					if (this.dom[param][value] !== undefined) return;
					// Grab all the value's controls at a moment
					this.dom[param][value] = container.find('[data-'+param+'="'+value+'"]');
					if ($(elm).hasClass('active')) this.active[param].include(value);
					this.dom[param][value].on('click', function(){
						this.toggle(param, value);
					}.bind(this));
				}.bind(this));
			}.bind(this));
		},

		/**
		 * Detach controls inside the container element
		 * @param container
		 */
		detach: function(container)
		{},

		toggle: function(param, value)
		{
			var singles = [];
			$.each(this.options.params, function(index, param){
				if (this.active[param].length < 2) singles.push(param);
			}.bind(this));
			/**
			 * Toggling radio button
			 */
			if (singles.length == 1 && singles.contains(param)){
				// Clicked on the active value. Do nothing.
				if (this.active[param].contains(value)) return;
				this.dom[param][this.active[param][0]].removeClass('active').removeClass('single');
				this.dom[param][value].addClass('active').addClass('single');
				this.active[param] = [value];
			}
			/**
			 * Toggling checkbox OFF
			 */
			else if (this.active[param].contains(value)){
				// Clicked on the only active checkbox. Do nothing.
				if (singles.contains(param)) return;
				// Turn off
				this.dom[param][value].removeClass('active');
				this.active[param].erase(value);
				if (this.active[param].length == 1){
					if (singles.length == 1) $(this.dom.wrapper[singles[0]]).removeClass('type_radio').addClass('type_checkbox');
					this.dom[param][this.active[param][0]].addClass('single');
				}
			}
			/**
			 * Toggling checkbox ON
			 */
			else{
				if (singles.contains(param)){
					singles.erase(param);
					if (singles.length == 1) $(this.dom.wrapper[singles[0]]).removeClass('type_checkbox').addClass('type_radio');
					this.dom[param][this.active[param][0]].removeClass('single');
				}

				this.active[param].push(value);

				// check for max
				if (this.options.seriesMax != null && this.options.seriesMax < this.active[param].length) {
					this.dom[param][this.active[param][0]].removeClass('active');
					this.active[param].erase(this.active[param][0]);
				}
				this.dom[param][value].addClass('active');
			}
			this.render();
		},

		render: function()
		{
			var compare = (this.active.rate.length > 1) ? 'rate' : ((this.active.company.length > 1) ? 'company' : false),
				series = [],
				i;

			// Grab units by the way
			this.active.units = [];

			$.each(this.active.rate, function(index, rate){
				this.active.units.include(this.data.rate[rate].units[0]);
			}.bind(this));

			// Check conditions
			var nextGroup = null;
			// Using this kind of iteration, because $.each iterates backwards in ie8
			for (var index = 0; index < 5; index++){
				if (nextGroup !== null) break;
				try {
					if (this.options.groups[index].condition.apply(this, [this.active]) === true){
						nextGroup = index;
					}
				}catch(e){ console.error(e); }
			}
			if (nextGroup === null) console.error('None of the groups suits present data');

			// Redraw controls
			var groupOpt = this.options.groups[nextGroup];
			Object.each(this.dom.controls, function(elm, key){
				if (groupOpt.controls[key] !== undefined){
					// Check for valid values
					$(elm).css('display', 'block').unbind('click').bind('click', groupOpt.controls[key].onClick.bind(this));
				}else{
					$(elm).css('display', 'none');
				}
			}, this);

			if (nextGroup === this._group){
				this._renderChart(this._chartType);
			}else{
				this._group = nextGroup;
				this._renderChart(groupOpt.charts[0]);
			}

			this.fireEvent('render');
		},

		_renderChart: function(chartType)
		{
			// Create local copy of active ids to be sorted
			var active = $.extend({}, this.active),
			// Only for highchart api functions
				that = this;
			$.each(active, function(index){active[index].sort();});

			var opt = Object.clone(this.options.charts[chartType]),
				groupOpt = this.options.groups[this._group],
				cats = active[opt.axis].map(function(val){ return this.data[opt.axis][val].title[this.options.lang]; }.bind(this)),
				cur = {};

			// Update controls
			$.each(groupOpt.controls, function(key, cn){
				if (cn.activeOn !== undefined){
					if (cn.activeOn.attempt([active, opt], this) === true){
						this.dom.controls[key].addClass('active');
					}else{
						this.dom.controls[key].removeClass('active');
					}
				}
				if (cn.showOn !== undefined){
					if (cn.showOn.attempt([active, opt], this) === true){
						this.dom.controls[key].css('display', 'block');
					}else{
						this.dom.controls[key].css('display', 'none');
					}
				}
			}.bind(this));

			// Chart options for HighChart object, API: http://api.highcharts.com/highcharts#chart
			var options = {
				chart: {
					renderTo: this.dom.chart.get(0),
					defaultSeriesType: opt.type,
					spacingTop: (this.options.dataLabels && (opt.type == 'line' || opt.type == 'column')) ? ((this.options.spacingTop != 10) ? this.options.spacingTop : 30) : this.options.spacingTop,
					spacingRight: this.options.spacingRight,
					spacingBottom: this.options.spacingBottom,
					spacingLeft: this.options.spacingLeft,
					plotBackgroundColor: 'none',
					plotBorderWidth: 0,
					plotShadow: false
				},
				xAxis: {categories: cats, title: {text: ''},
					tickLength: this.options.tickLengthX,
					labels: {
						rotation: this.options.rotationLabelsX,
						y: this.options.labelsYX,
						style: this.options.labelsXStyle
					}
				},
				yAxis: {title: {text: (this.options.hideYAxis) ? null : ''},
					labels: {
						enabled: !this.options.hideYAxis,
						formatter: function(){ return this.value; }
					},
					showEmpty: false,
					tickLength: this.options.tickLengthY,
					gridLineWidth: this.options.gridLineWidthY
				},
				credits: { enabled: false },
				title: {text: ''},
				legend: {enabled: this.options.legend},
				series: [],
				exporting: {buttons: {contextButton: {enabled: false}}},
				periodsFormat: function (value){ return this._periodsFormat(value) }.bind(this)
			};

			// Pie chart is separated in a different group as it has its own unique data structure
			if (opt.type == 'pie')
			{
				var rad = parseInt(this.options.pieRadius / active[opt.vary].length);
				cur[opt.fixed] = active[opt.fixed][0];

				this._countColors(active[opt.axis]);

				$.each(active[opt.vary], function(varyOrder, vary){
					// Count total sum
					var totalSum = 0,
						serie = [],
					// Compare positive values (will be switched to false, if all values are negative)
						positiveVals = true;
					cur[opt.vary] = vary;
					$.each(active[opt.axis], function(axisOrder, axis){
						cur[opt.axis] = axis;
						totalSum += Math.max(0, this.data.values[cur.company][cur.rate][cur.period]);
					}.bind(this));
					if (totalSum == 0){
						// In case of negative values
						$.each(active[opt.axis], function(axisOrder, axis){
							cur[opt.axis] = axis;
							totalSum += Math.abs(Math.min(0, this.data.values[cur.company][cur.rate][cur.period]));
						}.bind(this));
						positiveVals = false;
					}
					$.each(active[opt.axis], function(axisOrder, axis){
						cur[opt.axis] = axis;
						var data = {
								axisVal: axis,
								absY: this.data.values[cur.company][cur.rate][cur.period],
								unit: this.data.unit[this.data.rate[cur.rate].units[0]].title[this.options.lang],
								color: this.data[opt.axis][axis].color || this._colors[axis]
							},
							title = this.data[opt.axis][axis].title[this.options.lang];
						data.y = Math.abs(Math[positiveVals?'max':'min'](this.data.values[cur.company][cur.rate][cur.period], 0));
						data.name = title+' ('+(data.y/totalSum*100).toFixed(2)+'%)';
						serie.push(data);
					}.bind(this));
					options.series.push({
						name: this.data[opt.fixed][cur[opt.fixed]].title[this.options.lang]+', '+this.data[opt.vary][vary].title[this.options.lang],
						size: (rad*(varyOrder+1))+'%',
						innerSize: (rad*varyOrder)+'%',
						type: 'pie',
						data: serie,
						fixedVal: cur[opt.fixed],
						varyVal: vary,
						dataLabels: {
							enabled: (varyOrder == active[opt.vary].length - 1)
						}
					});
				}.bind(this));
				options.tooltip = {
					formatter: function(){
						return this.series.name +': <b>'+options.periodsFormat(this.point.absY)+' '+this.point.unit+'</b>';
					}
				};
			}
			else if (opt.type == 'stackedbar')
			{
				var cmpWith = opt.compare,
					cmpBy = (opt.vary == cmpWith) ? opt.axis : opt.vary,
					baseVals = {},
					prevVals = {},
					sums = {};
				cur[opt.fixed] = active[opt.fixed][0];

				this._countColors(active[opt.axis]);

				// Count sums
				$.each(active[opt.vary], function(varyOrder, vary){
					cur[opt.vary] = vary;
					$.each(active[opt.axis], function(axisOrder, axis){
						cur[opt.axis] = axis;
						sums[axis] = (sums[axis] || 0) + Math.max(0, this.data.values[cur.company][cur.rate][cur.period]);
					}.bind(this));
				}.bind(this));

				$.each(active[opt.vary], function(varyOrder, vary){
					var serie = [];
					cur[opt.vary] = vary;

					$.each(active[opt.axis], function(axisOrder, axis){
						cur[opt.axis] = axis;
						var y,
							firstValue = prevVals[cur[cmpBy]] === undefined || baseVals[cur[cmpBy]] === false;
						// Grab basic values that will be marked as 100% (nulls won't be grabbed)
						if (firstValue) baseVals[cur[cmpBy]] = this.data.values[cur.company][cur.rate][cur.period] || false;
						if (this.data.values[cur.company][cur.rate][cur.period] !== null && sums[axis] != 0){
							// Normal point
							y = (this.data.values[cur.company][cur.rate][cur.period]/sums[axis]*100);
						}else{
							// Missing point
							y = null;
						}
						// Fix stacked bar render for negative values
						if (y < 0) y = null;

						serie.push({
							name: this.data[opt.axis][axis].title[this.options.lang],
							absY: this.data.values[cur.company][cur.rate][cur.period],
							cmpVal: firstValue ? this.data.values[cur.company][cur.rate][cur.period-1] : (prevVals[cur[cmpBy]] || baseVals[cur[cmpBy]]),
							unit: this.data.unit[this.data.rate[cur.rate].units[0]].title[this.options.lang],
							axisVal: axis,
							y: y
						});
						// When compared with period use the previous one
						if (baseVals[cur[cmpBy]] === undefined || cmpWith == 'period') prevVals[cur[cmpBy]] = this.data.values[cur.company][cur.rate][cur.period];
					}.bind(this));
					options.series.push({
						name: this.data[opt.vary][vary].title[this.options.lang],
						data: serie,
						fixedVal: cur[opt.fixed],
						varyVal: vary,
						color: this.data[opt.vary][varyOrder].color || this._colors[vary]
					});
				}.bind(this));
				// Stacked bar type
				options.chart.defaultSeriesType = 'bar';
				options.plotOptions = {bar: {stacking: 'normal'}};
				options.tooltip = {
					formatter: function(){
						var result = this.series.name +': <b>'+ options.periodsFormat(this.point.absY) +' '+this.point.unit+'</b>';
						if (this.point.cmpVal !== false && this.point.cmpVal != 0){
							var diff = (this.point.absY - this.point.cmpVal) * 100 / this.point.cmpVal;
							result += ' ('+((diff<0)?'':'+')+diff.toFixed(2)+'%)';
						}
						return result;
					}
				};
				options.yAxis.labels.formatter = function(){ return this.value + ' %'; };
			}
			else if (opt.measure == 'absolute')
			{
				var cmpWith = opt.compare,
					cmpBy = (opt.vary == cmpWith) ? opt.axis : opt.vary,
					prevVals = {};
				cur[opt.fixed] = active[opt.fixed][0];

				this._countColors(active[opt.vary]);

				$.each(active[opt.vary], function(varyOrder, vary){
					var serie = [];
					cur[opt.vary] = vary;
					Array.each(active[opt.axis], function(axis, axisOrder){
						cur[opt.axis] = axis;
						var firstValue = prevVals[cur[cmpBy]] === undefined,
							pt = {
								y: this.data.values[cur.company][cur.rate][cur.period],
								absY: this.data.values[cur.company][cur.rate][cur.period],
								cmpVal: firstValue ? this.data.values[cur.company][cur.rate][cur.period-1] : prevVals[cur[cmpBy]],
								unit: this.data.unit[this.data.rate[cur.rate].units[0]].title[this.options.lang],
								axisVal: axis
							};

						// Fix stacked bar render for negative values
						if (opt.type == 'stackedbar') pt.y = Math.max(0, pt.y);

						// Add note
						if (this.data.notes !== undefined
							&& this.data.notes[cur.company] !== undefined
							&& this.data.notes[cur.company][cur.rate] !== undefined
							&& this.data.notes[cur.company][cur.rate][cur.period] !== undefined){
							pt.marker = {symbol: 'url(img/information.png)'};
							pt.note = this.data.notes[cur.company][cur.rate][cur.period];
							pt.cursor = 'pointer';
							pt.events = {
								click: function() {
									//alert(this.note);
//								hs.htmlExpand(null, {
//									pageOrigin: {x: this.pageX, y: this.pageY},
//									headingText: this.series.name,
//									maincontentText: Highcharts.dateFormat('%A, %b %e, %Y', this.x) +':<br/> '+this.y +' visits',
//									width: 200
//								});
								}
							};
						}
						// Add point to serie
						serie.push(pt);
						if (cmpWith == 'period') prevVals[cur[cmpBy]] = this.data.values[cur.company][cur.rate][cur.period];
					}, this);
					options.series.push({
						name: this.data[opt.vary][vary].title[this.options.lang],
						data: serie,
						fixedVal: cur[opt.fixed],
						varyVal: vary,
						color: this.data[opt.vary][varyOrder].color || this._colors[vary]
					});
				}.bind(this));
				// Plot options
				options.plotOptions = {
					line: {dataLabels: {enabled: this.options.dataLabels, formatter: function(){ return options.periodsFormat(this.point.absY); }}},
					column: {
						dataLabels: {
							enabled: this.options.dataLabels,
							rotation: this.options.rotationLabelsY,
							x: 2,
							y: this.options.labelsYY,
							formatter: function(){ return options.periodsFormat(this.point.absY);},
							style: this.options.labelsStyle
						}
					}
				};
				options.tooltip = {
					formatter: function(){
						var result = this.series.name +': <b>'+ options.periodsFormat(this.point.absY) +' '+this.point.unit+'</b>',
							diff;
						if (this.point.cmpVal !== null && this.point.cmpVal != 0){
							if (this.point.unit == '%'){
								diff = this.point.absY - this.point.cmpVal;
								result += ' ('+((diff<0)?'':'+')+diff.toFixed(2)+' '+that.options.i18n[that.options.lang].pp+')';
							}else{
								diff = (this.point.absY - this.point.cmpVal) * 100 / this.point.cmpVal;
								result += ' ('+((diff<0)?'':'+')+diff.toFixed(2)+'%)';
							}
						}
						return result;
					}
				};
				options['yAxis']['title'] = {
					text: (this.options.hideYAxis) ? null : this.data.unit[this.data.rate[cur.rate].units[0]].title[this.options.lang],
					align: 'high'
				};

				// Workaround for the highcharts formatter API
				options.yAxis.labels.formatter = function(value){ return that._periodsFormat(this.value); };
			}
			else if (opt.measure == 'relative')
			{
				//
				var cmpWith = opt.compare,
				// Which param will be varied for comparison
					cmpBy = (opt.vary == cmpWith) ? opt.axis : opt.vary,
				// First in-a-group values, that will define 100% on y-axis
					baseVals = {},
				// Previous in-a-group value to count relative difference from it
					prevVals = {};
				cur[opt.fixed] = active[opt.fixed][0];

				this._countColors(active[opt.vary]);

				$.each(active[opt.vary], function(varyOrder, vary){
					var data = [];
					cur[opt.vary] = vary;
					$.each(active[opt.axis], function(axisOrder, axis){
						cur[opt.axis] = axis;
						var y,
							firstValue = baseVals[cur[cmpBy]] === undefined;
						// Grab basic values that will be marked as 100% (nulls won't be grabbed)
						if (firstValue && this.data.values[cur.company][cur.rate][cur.period] != 0){
							baseVals[cur[cmpBy]] = this.data.values[cur.company][cur.rate][cur.period] || undefined;
						}
						if (this.data.values[cur.company][cur.rate][cur.period] !== null
							// First value cannot be zero
							&& (!firstValue || this.data.values[cur.company][cur.rate][cur.period] != 0)){
							// Normal point
							y = (this.data.values[cur.company][cur.rate][cur.period]/baseVals[cur[cmpBy]]*100);
							data.push({
								name: this.data[opt.axis][axis].title[this.options.lang],
								absY: this.data.values[cur.company][cur.rate][cur.period],
								cmpVal: firstValue ? this.data.values[cur.company][cur.rate][cur.period-1] : (prevVals[cur[cmpBy]] || baseVals[cur[cmpBy]]),
								unit: this.data.unit[this.data.rate[cur.rate].units[0]].title[this.options.lang],
								axisVal: axis,
								y: y
							});

							// When compared with period use the previous one
							if (baseVals[cur[cmpBy]] === undefined || cmpWith == 'period') prevVals[cur[cmpBy]] = this.data.values[cur.company][cur.rate][cur.period];
						}else{
							// Missing point
							y = null;
							data.push(null);
						}
					}.bind(this));
					options.series.push({
						name: this.data[opt.vary][vary].title[this.options.lang],
						data: data,
						fixedVal: cur[opt.fixed],
						varyVal: vary,
						color: this.data[opt.vary][varyOrder].color || this._colors[vary]
					});
				}.bind(this));
				// Plot options
				options.plotOptions = {
					line: {dataLabels: {enabled: this.options.dataLabels, formatter: function(){ return options.periodsFormat(this.point.absY); }}},
					column: {
						dataLabels: {
							enabled: this.options.dataLabels,
							rotation: 315,
							x: 2,
							y: -15,
							formatter: function(){ return options.periodsFormat(this.point.absY);}
						}
					}
				};
				options.tooltip = {
					formatter: function(){
						var result = this.series.name +': <b>'+ options.periodsFormat(this.point.absY) +' '+this.point.unit+'</b>',
							diff;
						if (this.point.cmpVal !== false && this.point.cmpVal != 0){
							if (this.point.unit == '%'){
								diff = this.point.absY - this.point.cmpVal;
								result += ' ('+((diff<0)?'':'+')+diff.toFixed(2)+' '+that.options.i18n[that.options.lang].pp+')';
							}else{
								diff = (this.point.absY - this.point.cmpVal) * 100 / this.point.cmpVal;
								result += ' ('+((diff<0)?'':'+')+diff.toFixed(2)+'%)';
							}
						}
						return result;
					}
				};
				options['yAxis']['title'] = {text: (this.options.hideYAxis) ? null : '%', align: 'high'};
//			options.yAxis.labels.formatter = function(){ return this.value + ' %'; };
			}

			// TODO Remove force redraw when color duplication and wrong removal will be fixed
			if (true || chartType !== this._chartType || opt.redraw){
				// Completely redraw chart
				try{
					this.chart.destroy();
				}catch(e){ console.log(e); }
				try {
					this.chart = new Highcharts.Chart(options);
				}catch(e){ console.log(e); }
			}else{
				// Update chart
				this._updateChart(options);
			}

			// Update value for chart export to prevent circular traversing

			// Collect data for export-to-excel function
			if ('toJSON' in $){
				delete options.chart.renderTo;
				if (options.exporting !== undefined) delete options.exporting;
				// Pass current used language
				options.lang = this.options.lang;
				this.dom.xlsexport.attr('value', $.toJSON(options));
			}

			this._chartType = chartType;
		},

		/**
		 * Update chart instead of complete redraw
		 *
		 * @param {Object} options
		 * @private
		 */
		_updateChart: function(options)
		{
			var i, l, oldSeries = {}, newSeries = {};

			// Old
			this.chart.options.series.each(function(serie, index){
				oldSeries[serie.varyVal] = serie;

				// Store index for future usage
				oldSeries[serie.varyVal].index = index;
			}, this);

			// New
			options.series.each(function(serie, index){
				newSeries[serie.varyVal] = serie;

				// Store index for future usage
				newSeries[serie.varyVal].index = index;
			}, this);

			// Update xAxis categories
			if ( ! ($(options.xAxis.categories).not(this.chart.axes[0].categories).length == 0 && $(this.chart.axes[0].categories).not(options.xAxis.categories).length == 0)) {
				this.chart.axes[0].setCategories(options.xAxis.categories);
			}

			// Add nessessary series
			var createSeries = [];
			Object.each(newSeries, function(serie, varyVal){
				if (oldSeries[varyVal] === undefined) createSeries.push(varyVal);
			}, this);

			// Prepare unnessesary series for deletion
			var deleteSeries = [];
			Object.each(oldSeries, function(serie, varyVal){
				if ( ! createSeries.contains(varyVal)){
					if(newSeries[varyVal] === undefined) {
						deleteSeries.push(varyVal);
					} else {
						this.chart.series[serie.index].setData(newSeries[varyVal].data, false);
					}
				}
			}, this);

			// Delete
			deleteSeries.each(function(varyVal, index){
				this.chart.series[oldSeries[varyVal].index].remove(false);
			}, this);

			// Create
			createSeries.each(function(varyVal){
				this.chart.addSeries(newSeries[varyVal], false);
			}, this);

			this.chart.options.series = jQuery.map(newSeries, function (val, key) { return val; });

			// Update axis formatter
			this.chart.axes[1].options.labels.formatter = options.yAxis.labels.formatter;
			try {
				this.chart.redraw();
			} catch (e) {}
		},

		/**
		 * Update predefined colors schema in local _colors array
		 * @private
		 */
		_countColors: function(baseValues)
		{
			// Keep the original array clean
			var values = $.extend([], baseValues),
				usedColors = [];

			$.each(this._colors, function(key, color){
				if ($.inArray(parseInt(key), values) == -1){
					// Remove unused colors
					delete this._colors[key];
				}else{
					// Remain this color and bind it for present key
					usedColors.push(color);
				}
			}.bind(this));

			// Add new colors
			$.each(values, function(index1, key){
				// Proceed only for new keys
				if (this._colors[key] !== undefined) return;
				// Grab first not-used color
				var newColor = $.grep(this.options.colors, function(color){ return $.inArray(color, usedColors) == -1; })[0];
				usedColors.push(newColor);
				this._colors[key] = newColor;
			}.bind(this));
		},

		/**
		 * Format periods on lang preferences
		 * @private
		 */
		_periodsFormat: function (x) {
			if (x == undefined) return x;
			var delimiter = '',
				decimals = '.',
				parts = x.toString().split(".");

			if (this.options.lang == 'en'){
				delimiter = ',';
				decimals = '.';
			}
			if (this.options.lang == 'ru'){
				delimiter = ' ';
				decimals = ',';
			}

			return parts[0].replace(/\B(?=(\d{3})+(?=$))/g, delimiter) + (parts[1] ? decimals + parts[1] : "");
		}

	});
}(jQuery);
